@@ -1,18 +1,19 @@
 """
-    flask_security
-    ~~~~~~~~~~~~~~
+flask_security
+~~~~~~~~~~~~~~
 
-    Flask-Security is a Flask extension that aims to add comprehensive security
-    to Flask applications.
+Flask-Security is a Flask extension that aims to add comprehensive security
+to Flask applications.
 
-    :copyright: (c) 2012-2019 by Matt Wright.
-    :copyright: (c) 2019-2024 by J. Christopher Wagner.
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2012-2019 by Matt Wright.
+:copyright: (c) 2019-2025 by J. Christopher Wagner.
+:license: MIT, see LICENSE for more details.
 """
 
 # flake8: noqa: F401
 from .changeable import admin_change_password
 from .change_email import ChangeEmailForm
+from .change_username import ChangeUsernameForm
 from .core import (
     Security,
     RoleMixin,
@@ -45,20 +46,22 @@ from .decorators import (
     unauth_csrf,
 )
 from .forms import (
-    Form,
     ChangePasswordForm,
+    ConfirmRegisterForm,
+    Form,
     ForgotPasswordForm,
     LoginForm,
-    RegisterForm,
-    ResetPasswordForm,
     PasswordlessLoginForm,
-    ConfirmRegisterForm,
+    RegisterForm,
+    RegisterFormV2,
+    ResetPasswordForm,
     SendConfirmationForm,
     TwoFactorRescueForm,
     TwoFactorSetupForm,
     TwoFactorVerifyCodeForm,
-    VerifyForm,
     unique_identity_attribute,
+    UsernameRecoveryForm,
+    VerifyForm,
 )
 from .mail_util import MailUtil, EmailValidateException
 from .oauth_glue import OAuthGlue
@@ -87,6 +90,8 @@ from .signals import (
     user_confirmed,
     user_registered,
     user_not_registered,
+    username_recovery_email_sent,
+    username_changed,
     us_security_token_sent,
     us_profile_changed,
     wan_deleted,
@@ -139,4 +144,4 @@ from .webauthn import (
 )
 from .webauthn_util import WebauthnUtil
 
-__version__ = "5.5.0"
+__version__ = "5.6.0"
