@@ -1,31 +1,31 @@
 """
-    flask_security.unified_signin
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+flask_security.unified_signin
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Flask-Security Unified Signin module
+Flask-Security Unified Signin module
 
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2019-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 
-    This implements a unified sign in endpoint - allowing
-    authentication via identity and passcode - where identity is configured
-    via SECURITY_USER_IDENTITY_ATTRIBUTES, and allowable passcodes are
-    one of US_ENABLED_METHODS.
+This implements a unified sign in endpoint - allowing
+authentication via identity and passcode - where identity is configured
+via SECURITY_USER_IDENTITY_ATTRIBUTES, and allowable passcodes are
+one of US_ENABLED_METHODS.
 
-    Finish up:
-    - we should be able to add a phone number as part of setup even w/o any METHODS -
-      i.e. to allow login with any identity (phone) and a password.
+Finish up:
+- we should be able to add a phone number as part of setup even w/o any METHODS -
+  i.e. to allow login with any identity (phone) and a password.
 
-    Consider/Questions:
-    - Allow registering/confirming with just a phone number - this likely would require
-      a new register/confirm endpoint in order to implement verification.
-    - Right now ChangePassword won't work - it requires an existing password - so
-      if the user doesn't have one - can't change it. However ForgotPassword will in
-      fact allow the user to add a password. Is that sufficient?
-    - This also means that there is no way to REMOVE your password once it is setup,
-      although user can register without one.
-    - separate code validation times for SMS, email, authenticator?
-    - token versus code versus passcode? Confusing terminology.
+Consider/Questions:
+- Allow registering/confirming with just a phone number - this likely would require
+  a new register/confirm endpoint in order to implement verification.
+- Right now ChangePassword won't work - it requires an existing password - so
+  if the user doesn't have one - can't change it. However ForgotPassword will in
+  fact allow the user to add a password. Is that sufficient?
+- This also means that there is no way to REMOVE your password once it is setup,
+  although user can register without one.
+- separate code validation times for SMS, email, authenticator?
+- token versus code versus passcode? Confusing terminology.
 
 """
 
@@ -55,7 +55,7 @@ from .forms import (
     _setup_methods_xlate,
     Form,
     NextFormMixin,
-    Required,
+    RequiredLocalize,
     build_form_from_request,
     build_form,
     form_errors_munge,
@@ -188,6 +188,7 @@ class _UnifiedPassCodeForm(Form):
             # Since we have a unique totp_secret for each method - we
             # can figure out which mechanism was used.
             # Note that password check requires a string (not int or None)
+            assert isinstance(self.passcode.errors, list)
             passcode = self.passcode.data
             if not passcode:
                 self.passcode.errors.append(get_message("INVALID_PASSWORD_CODE")[0])
@@ -218,6 +219,7 @@ class _UnifiedPassCodeForm(Form):
             return True
         elif self.submit_send_code.data:
             # Send a code - chosen_method must be valid
+            assert isinstance(self.chosen_method.errors, list)
             cm = self.chosen_method.data
             if cm not in cv("US_ENABLED_METHODS"):
                 self.chosen_method.errors.append(
@@ -244,7 +246,7 @@ class UnifiedSigninForm(_UnifiedPassCodeForm, NextFormMixin):
 
     identity = StringField(
         get_form_field_label("identity"),
-        validators=[Required()],
+        validators=[RequiredLocalize()],
     )
     remember = BooleanField(get_form_field_label("remember_me"))
 
@@ -263,6 +265,7 @@ class UnifiedSigninForm(_UnifiedPassCodeForm, NextFormMixin):
 
         # Can't authenticate nor get a code if still required confirmation.
         self.requires_confirmation = requires_confirmation(self.user)
+        assert isinstance(self.identity.errors, list)
         if self.requires_confirmation:
             self.identity.errors.append(get_message("CONFIRMATION_REQUIRED")[0])
             return False
@@ -322,6 +325,10 @@ class UnifiedSigninSetupForm(Form):
         if not super().validate(**kwargs):
             return False
 
+        assert isinstance(self.chosen_method.errors, list)
+        assert isinstance(self.phone.errors, list)
+        assert isinstance(self.delete_method.errors, list)
+
         if not self.chosen_method.data and not self.delete_method.data:
             self.form_errors.append(get_message("API_ERROR")[0])
             return False
@@ -333,6 +340,9 @@ class UnifiedSigninSetupForm(Form):
                 return False
 
             if self.chosen_method.data == "sms":
+                if not self.phone.data:
+                    self.phone.errors.append(get_message("PHONE_INVALID")[0])
+                    return False
                 msg = _security._phone_util.validate_phone_number(self.phone.data)
                 if msg:
                     self.phone.errors.append(msg)
@@ -371,12 +381,12 @@ class UnifiedSigninSetupValidateForm(Form):
         get_form_field_label("passcode"),
         render_kw={
             "autocomplete": "one-time-code",
-            "inputtype": "numeric",
+            "type": "text",
             "pattern": "[0-9]*",
         },
-        validators=[Required()],
+        validators=[RequiredLocalize()],
     )
-    submit = SubmitField(get_form_field_label("submitcode"))
+    submit = SubmitField(get_form_field_label("submitcode"), id="submit-code")
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -385,6 +395,8 @@ class UnifiedSigninSetupValidateForm(Form):
         if not super().validate(**kwargs):
             return False
 
+        assert isinstance(self.passcode.errors, list)
+        assert self.passcode.data is not None  # RequiredLocalize validator
         if not _security._totp_factory.verify_totp(
             token=self.passcode.data,
             totp_secret=self.totp_secret,
@@ -429,6 +441,7 @@ def us_signin_send_code() -> ResponseValue:
     if form.validate_on_submit():
         msg = _send_code_helper(form, True)
         if msg:
+            assert isinstance(form.chosen_method.errors, list)
             form.chosen_method.errors.append(msg)
 
         if _security._want_json(request):
@@ -500,6 +513,7 @@ def us_verify_send_code() -> ResponseValue:
     if form.validate_on_submit():
         msg = _send_code_helper(form, False)
         if msg:
+            assert isinstance(form.chosen_method.errors, list)
             form.chosen_method.errors.append(msg)
 
         if _security._want_json(request):
@@ -829,11 +843,10 @@ def us_setup() -> ResponseValue:
             # N.B. totp (totp_secret) is actually encrypted - so it seems safe enough
             # to send it to the user.
             # Only check phone number if SMS (see form validate)
-            phone_number = (
-                _security._phone_util.get_canonical_form(form.phone.data)
-                if add_method == "sms"
-                else None
-            )
+            phone_number = None
+            if add_method == "sms":
+                assert form.phone.data is not None
+                phone_number = _security._phone_util.get_canonical_form(form.phone.data)
             state = {
                 "totp_secret": totp,
                 "chosen_method": add_method,
@@ -846,6 +859,7 @@ def us_setup() -> ResponseValue:
             )
             if msg:
                 # sending didn't work.
+                assert isinstance(form.chosen_method.errors, list)
                 form.chosen_method.errors.append(msg)
                 if _security._want_json(request):
                     # Not authenticated yet - so don't send any user info.
