@@ -1,12 +1,12 @@
 """
-    test_misc
-    ~~~~~~~~~~~
+test_misc
+~~~~~~~~~~~
 
-    Lots of tests
+Lots of tests
 
-    :copyright: (c) 2012 by Matt Wright.
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2012 by Matt Wright.
+:copyright: (c) 2019-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 """
 
 from datetime import timedelta
@@ -19,6 +19,7 @@ import typing as t
 
 import pytest
 
+from itsdangerous import BadTimeSignature
 from wtforms.validators import DataRequired, Length
 
 from tests.test_utils import (
@@ -33,6 +34,7 @@ from tests.test_utils import (
     logout,
     populate_data,
     reset_fresh,
+    get_form_input,
 )
 from tests.test_webauthn import HackWebauthnUtil, reg_2_keys
 
@@ -48,7 +50,8 @@ from flask_security.forms import (
     PasswordField,
     PasswordlessLoginForm,
     RegisterForm,
-    Required,
+    RegisterFormV2,
+    RequiredLocalize,
     ResetPasswordForm,
     SendConfirmationForm,
     StringField,
@@ -67,6 +70,7 @@ from flask_security.utils import (
     uia_phone_mapper,
     verify_hash,
 )
+from flask_security.core import _get_serializer
 
 if t.TYPE_CHECKING:  # pragma: no cover
     from flask.testing import FlaskClient
@@ -113,6 +117,9 @@ def test_register_blueprint_flag(app, sqlalchemy_datastore):
         {"username": {"mapper": lambda x: x}},
     ]
 )
+@pytest.mark.filterwarnings(
+    "ignore:.*The RegisterForm is deprecated.*:DeprecationWarning"
+)
 def test_basic_custom_forms(app, sqlalchemy_datastore):
     class MyLoginForm(LoginForm):
         username = StringField("My Login Username Field")
@@ -169,6 +176,7 @@ def test_basic_custom_forms(app, sqlalchemy_datastore):
 
 @pytest.mark.registerable()
 @pytest.mark.confirmable()
+@pytest.mark.filterwarnings("ignore:.*The ConfirmRegisterForm.*:DeprecationWarning")
 def test_confirmable_custom_form(app, sqlalchemy_datastore):
     app.config["SECURITY_REGISTERABLE"] = True
     app.config["SECURITY_CONFIRMABLE"] = True
@@ -358,7 +366,7 @@ def test_custom_forms_via_config(app, sqlalchemy_datastore):
     class MyLoginForm(LoginForm):
         email = StringField("My Login Email Address Field")
 
-    class MyRegisterForm(RegisterForm):
+    class MyRegisterForm(RegisterFormV2):
         email = StringField("My Register Email Address Field")
 
     app.config["SECURITY_LOGIN_FORM"] = MyLoginForm
@@ -368,7 +376,6 @@ def test_custom_forms_via_config(app, sqlalchemy_datastore):
     security.init_app(app)
 
     client = app.test_client()
-
     response = client.get("/login")
     assert b"My Login Email Address Field" in response.data
 
@@ -399,6 +406,7 @@ def test_custom_form_instantiator(app, client, get_message):
         def validate(self, **kwargs: t.Any) -> bool:
             if not super().validate(**kwargs):  # pragma: no cover
                 return False
+            assert isinstance(self.email.errors, list)
             if not self.myservice(self.email.data):
                 self.email.errors.append("Not happening")
                 return False
@@ -437,6 +445,7 @@ def test_custom_form_instantiator2(app, client, get_message):
         def validate(self, **kwargs: t.Any) -> bool:
             if not super().validate(**kwargs):  # pragma: no cover
                 return False
+            assert isinstance(self.email.errors, list)
             if not self.myservice(self.email.data):
                 self.email.errors.append("Not happening")
                 return False
@@ -481,7 +490,7 @@ def test_custom_form_setting(app, sqlalchemy_datastore):
 
 def test_form_required(app, sqlalchemy_datastore):
     class MyLoginForm(LoginForm):
-        myfield = StringField("My Custom Field", validators=[Required()])
+        myfield = StringField("My Custom Field", validators=[RequiredLocalize()])
 
     app.config["SECURITY_LOGIN_FORM"] = MyLoginForm
 
@@ -501,7 +510,9 @@ def test_form_required_local_message(app, sqlalchemy_datastore):
     msg = "hi! did you forget me?"
 
     class MyLoginForm(LoginForm):
-        myfield = StringField("My Custom Field", validators=[Required(message=msg)])
+        myfield = StringField(
+            "My Custom Field", validators=[RequiredLocalize(message=msg)]
+        )
 
     app.config["SECURITY_LOGIN_FORM"] = MyLoginForm
 
@@ -756,7 +767,7 @@ def test_per_request_xlate(app, client):
     response = client.get("/change", follow_redirects=True)
     assert response.status_code == 200
     assert b"Nouveau mot de passe" in response.data
-    assert b"<h1>Changer de mot de passe</h1>" in response.data
+    assert b"<h1>Changer le mot de passe</h1>" in response.data
 
     # try JSON
     response = client.post(
@@ -1139,7 +1150,7 @@ def test_verify_fresh(app, client, get_message):
 
     with capture_flashes() as flashes:
         response = client.get("/fresh", follow_redirects=True)
-        assert b"Please Reauthenticate" in response.data
+        assert b"Reauthenticate" in response.data
     assert flashes[0]["category"] == "error"
     assert flashes[0]["message"].encode("utf-8") == get_message(
         "REAUTHENTICATION_REQUIRED"
@@ -1152,12 +1163,12 @@ def test_verify_fresh(app, client, get_message):
 
     reset_fresh(client, app.config["SECURITY_FRESHNESS"])
     response = client.get(verify_url)
-    assert b"Please Reauthenticate" in response.data
+    assert b"Reauthenticate" in response.data
 
     response = client.post(
         verify_url, data=dict(password="not my password"), follow_redirects=False
     )
-    assert b"Please Reauthenticate" in response.data
+    assert b"Reauthenticate" in response.data
 
     response = client.post(
         verify_url, data=dict(password="password"), follow_redirects=False
@@ -1180,7 +1191,7 @@ def test_verify_fresh_json(app, client, get_message):
     assert response.json["response"]["reauth_required"]
 
     response = client.get("/verify")
-    assert b"Please Reauthenticate" in response.data
+    assert b"Reauthenticate" in response.data
 
     response = client.post(
         "/verify", json=dict(password="not my password"), headers=headers
@@ -1521,3 +1532,72 @@ def test_simplify_url():
     assert s == "/login"
     s = simplify_url("https:/myhost/profile", "https://localhost/login")
     assert s == "https://localhost/login"
+
+
+@pytest.mark.parametrize(
+    "verify_secret_key, verify_fallbacks, should_pass",
+    [
+        ("new_secret", [], False),  # Should fail - only new key
+        ("new_secret", ["old_secret"], True),  # Should pass - has fallback
+        ("old_secret", [], True),  # Should pass - using original key
+        ("wrong_secret", ["also_wrong"], False),  # Should fail - no valid keys
+    ],
+    ids=["new-key-only", "with-fallback", "original-key", "wrong-keys"],
+)
+def test_secret_key_fallbacks(app, verify_secret_key, verify_fallbacks, should_pass):
+    # Create token with original key
+    app.config["SECRET_KEY"] = "old_secret"
+    serializer = _get_serializer(app, "CONFIRM")
+    token = serializer.dumps({"data": "test"})
+
+    # Attempt verification with different key configurations
+    app.config["SECRET_KEY"] = verify_secret_key
+    app.config["SECRET_KEY_FALLBACKS"] = verify_fallbacks
+    serializer = _get_serializer(app, "CONFIRM")
+
+    if should_pass:
+        data = serializer.loads(token)
+        assert data["data"] == "test"
+    else:
+        with pytest.raises(BadTimeSignature):
+            serializer.loads(token)
+
+
+@pytest.mark.settings(username_enable=True)
+def test_custom_login_form(app, sqlalchemy_datastore, get_message):
+    # Test custom login form that deletes email and uses username only
+    # Also test that if app leave 'email' in as a user identity attribute we
+    # will ignore it
+    class MyLoginForm(LoginForm):
+        email = None
+
+    app.security = Security(
+        app,
+        datastore=sqlalchemy_datastore,
+        login_form=MyLoginForm,
+    )
+
+    populate_data(app)
+    client = app.test_client()
+
+    response = client.get("/login", follow_redirects=False)
+    assert not get_form_input(response, "email")
+
+    response = client.post(
+        "/login", json=dict(email="jill@lp.com", password="password")
+    )
+    assert response.status_code == 400
+    assert (
+        get_message("USER_DOES_NOT_EXIST")
+        == response.json["response"]["field_errors"][""][0].encode()
+    )
+
+    response = client.post("/login", json=dict(username="jill", password="password"))
+    assert response.status_code == 200
+
+
+@pytest.mark.settings(password_required=False)
+def test_password_required_setting(app, sqlalchemy_datastore):
+    with pytest.raises(ValueError) as vex:
+        Security(app=app, datastore=sqlalchemy_datastore)
+    assert "SECURITY_PASSWORD_REQUIRED can only be" in str(vex.value)
