@@ -1,11 +1,11 @@
 """
-    test_unified_signin
-    ~~~~~~~~~~~~~~~~~~~
+test_unified_signin
+~~~~~~~~~~~~~~~~~~~
 
-    Unified signin tests
+Unified signin tests
 
-    :copyright: (c) 2019-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2019-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 
 """
 
@@ -718,10 +718,10 @@ def test_verify_link_spa(app, client, get_message):
 
 
 def test_setup(app, clients, get_message):
-    client = clients
+    tcl = clients
     set_email(app)
-    us_authenticate(client)
-    response = client.get("us-setup")
+    us_authenticate(tcl)
+    response = tcl.get("us-setup")
     # Email should be in delete options since we just set that up.
     assert all(
         i in response.data
@@ -729,22 +729,24 @@ def test_setup(app, clients, get_message):
     )
 
     # test not supplying anything to do
-    response = client.post("us-setup", data=dict(phone="6505551212"))
+    response = tcl.post("us-setup", data=dict(phone="6505551212"))
     assert get_message("API_ERROR") in response.data
 
     # test missing phone
-    response = client.post("us-setup", data=dict(chosen_method="sms", phone=""))
+    response = tcl.post("us-setup", data=dict(chosen_method="sms", phone=""))
     assert response.status_code == 200
     assert get_message("PHONE_INVALID") in response.data
 
     # test invalid phone
-    response = client.post("us-setup", data=dict(chosen_method="sms", phone="555-1212"))
+    response = tcl.post(
+        "us-setup", data=dict(chosen_method="sms", phone="NOT-A-NUMBER")
+    )
     assert response.status_code == 200
     assert get_message("PHONE_INVALID") in response.data
     assert b"Enter code here to complete setup" not in response.data
 
     sms_sender = SmsSenderFactory.createSender("test")
-    response = client.post(
+    response = tcl.post(
         "us-setup", data=dict(chosen_method="sms", phone="650-555-1212")
     )
     assert response.status_code == 200
@@ -754,11 +756,11 @@ def test_setup(app, clients, get_message):
     verify_url = get_form_action(response, 1)
 
     # Try invalid code
-    response = client.post(verify_url, data=dict(passcode=12345), follow_redirects=True)
+    response = tcl.post(verify_url, data=dict(passcode=12345), follow_redirects=True)
     assert get_message("INVALID_PASSWORD_CODE") in response.data
 
     code = sms_sender.messages[0].split()[-1].strip(".")
-    response = client.post(verify_url, data=dict(passcode=code), follow_redirects=True)
+    response = tcl.post(verify_url, data=dict(passcode=code), follow_redirects=True)
     assert response.status_code == 200
     assert get_message("US_SETUP_SUCCESSFUL") in response.data
 
@@ -1021,7 +1023,7 @@ def test_verify(app, clients, get_message):
 
     response = client.get("us-setup", follow_redirects=True)
     form_response = response.data.decode("utf-8")
-    assert "Please Reauthenticate" in form_response
+    assert "Reauthenticate" in form_response
     send_code_url = get_form_action(response, 1)
 
     # Send unknown method
@@ -1494,7 +1496,7 @@ def test_tf(app, client, get_message):
         follow_redirects=True,
     )
     assert response.status_code == 200
-    message = b"Two-factor authentication adds an extra layer of security"
+    message = b"Two-Factor authentication adds an extra layer of security"
     assert message in response.data
     assert b"Set up using SMS" in response.data
 
@@ -1528,7 +1530,7 @@ def test_tf_link(app, client, get_message):
     magic_link = matcher.group(1)
     response = client.get(magic_link, follow_redirects=True)
     assert get_message("PASSWORDLESS_LOGIN_SUCCESSFUL") not in response.data
-    message = b"Two-factor authentication adds an extra layer of security"
+    message = b"Two-Factor authentication adds an extra layer of security"
     assert message in response.data
 
 
@@ -1699,6 +1701,9 @@ def test_replace_send_code(app, get_message):
     response = client.post("/us-signin/send-code", data=data, follow_redirects=True)
     assert b"Code has been sent" in response.data
 
+    with app.app_context():
+        db.engine.dispose()  # sqlite wants everything cleaned up
+
 
 @pytest.mark.settings(us_enabled_methods=["password"])
 def test_only_passwd(app, client, get_message):
@@ -1751,7 +1756,7 @@ def test_totp_generation(app, client, get_message):
         "us-setup", json=dict(chosen_method="authenticator"), headers=headers
     )
     assert response.status_code == 200
-    assert response.json["response"]["authr_issuer"] == "service_name"
+    assert response.json["response"]["authr_issuer"] == "tests"
     assert response.json["response"]["authr_username"] == "dave@lp.com"
     assert "authr_key" in response.json["response"]
 
