@@ -1,11 +1,11 @@
 """
-    test_webauthn
-    ~~~~~~~~~~~~~~~~~~~
+test_webauthn
+~~~~~~~~~~~~~~~~~~~
 
-    WebAuthn tests
+WebAuthn tests
 
-    :copyright: (c) 2021-2024 by J. Christopher Wagner (jwag).
-    :license: MIT, see LICENSE for more details.
+:copyright: (c) 2021-2025 by J. Christopher Wagner (jwag).
+:license: MIT, see LICENSE for more details.
 
 """
 
@@ -25,7 +25,7 @@ from tests.test_utils import (
     check_location,
     get_existing_session,
     get_form_action,
-    get_form_input,
+    get_form_input_value,
     is_authenticated,
     json_authenticate,
     logout,
@@ -203,8 +203,10 @@ def _register_start(
     return register_options, response_url
 
 
-def _register_start_json(client, name="testr1", usage="secondary"):
-    response = client.post("wan-register", json=dict(name=name, usage=usage))
+def _register_start_json(client, name="testr1", usage="secondary", csrf_token=None):
+    response = client.post(
+        "wan-register", json=dict(name=name, usage=usage, csrf_token=csrf_token)
+    )
     register_options = response.json["response"]["credential_options"]
     response_url = f'wan-register/{response.json["response"]["wan_state"]}'
     return register_options, response_url
@@ -232,6 +234,20 @@ def reg_2_keys(client):
     }
 
 
+def reg_first_key(client, csrf_token=None):
+    # Register a primary key - assumes already authenticated
+    # This can be used by other tests outside this module.
+    register_options, response_url = _register_start_json(
+        client, name="first", usage="first", csrf_token=csrf_token
+    )
+    response = client.post(
+        response_url,
+        json=dict(credential=json.dumps(REG_DATA_UV), csrf_token=csrf_token),
+    )
+    assert response.status_code == 200
+    return {"id": REG_DATA_UV["id"], "signin": SIGNIN_DATA_UV}
+
+
 def _signin_start(
     client,
     identity=None,
@@ -305,8 +321,8 @@ def test_basic(app, clients, get_message):
     response = clients.get("/wan-register")
     # default config allows for both primary and secondary usage
     # so form should have selector
-    assert get_form_input(response, "usage-0")
-    assert get_form_input(response, "usage-1")
+    assert get_form_input_value(response, "usage-0")
+    assert get_form_input_value(response, "usage-1")
 
     # post with no name
     response = clients.post("/wan-register", data=dict())
@@ -526,6 +542,11 @@ def test_bad_data_register(app, client, get_message):
     assert response.json["response"]["errors"][0].encode("utf-8") == get_message(
         "API_ERROR"
     )
+    response = client.post(response_url, json=dict(credential=""))
+    assert response.status_code == 400
+    assert response.json["response"]["errors"][0].encode("utf-8") == get_message(
+        "API_ERROR"
+    )
 
     # Now pass incorrect keys
     bad_register = copy.deepcopy(REG_DATA1)
@@ -574,6 +595,11 @@ def test_bad_data_signin(app, client, get_message):
     assert response.json["response"]["errors"][0].encode("utf-8") == get_message(
         "API_ERROR"
     )
+    response = client.post(response_url, json=dict(credential=""))
+    assert response.status_code == 400
+    assert response.json["response"]["errors"][0].encode("utf-8") == get_message(
+        "API_ERROR"
+    )
 
     # Now pass incorrect keys
     bad_signin = copy.deepcopy(SIGNIN_DATA1)
@@ -1677,7 +1703,7 @@ def test_login_next(app, client, get_message):
     )
     response_url = get_form_action(response)
 
-    next_loc = get_form_input(response, "next")
+    next_loc = get_form_input_value(response, "next")
     response = client.post(
         response_url,
         data=dict(credential=json.dumps(SIGNIN_DATA1), next=next_loc),
@@ -1739,7 +1765,7 @@ def test_async(app, client, get_message):
 )
 def test_csrf(app, client, get_message):
     response = client.get("/login")
-    csrf_token = get_form_input(response, "csrf_token")
+    csrf_token = get_form_input_value(response, "csrf_token")
     authenticate(client, csrf=True)
 
     register_options, response_url = _register_start(
@@ -1763,7 +1789,7 @@ def test_csrf(app, client, get_message):
     assert b"The CSRF tokens do not match." in response.data
 
     response = client.get("/wan-signin")
-    csrf_token = get_form_input(response, "csrf_token")
+    csrf_token = get_form_input_value(response, "csrf_token")
     signin_options, response_url = _signin_start(
         client, "matt@lp.com", csrf_token=csrf_token
     )
